<?php
namespace Folha\Logic\ControlePonto;
use Folha\Entities\GpeHordiah;
use Folha\Entities\GpeIntervaloshorario;
use Folha\Logic\ControlePonto\DateUtil;
use Folha\Logic\ControlePonto\FPA;
/**
 * Horario.
 *
 * @author  ingridmoreira
 */

class Horario{
    private $horario;
    private $horarioHistorico;
    private $intervalosHorario = array();
    private $indexHorario;
    private $endOfHorario;

    //Dia Ocorrência
    const DIA_ANTERIOR       = 'DA';
    const DIA_CORRENTE       = 'DC';
    const DIA_SEGUINTE       = 'DS';
    const FORCA_DIA_ANTERIOR = 'FA';
    const FORCA_DIA_CORRENTE = 'FC';
    const FORCA_DIA_SEGUINTE = 'FS';

    const FERIADO             = 'FERIADO';
    const IDTIPOHORARIO_TURNO = 'T';

    /**
     * Construtor.
     *
     * @author ingridmoreira
     */
    public function __construct($data, $horario, $tipoDiaCalend = null, $existeMarcacao = null){
        $this->horario = $horario;
        $dtCompetencia = FPA::getDtCompetencia();

        $where = "NRHORDIAM = ?
              AND NRORG = ?
              AND DTMESCOMPETENC IN (
                                     SELECT MAX(HDH.DTMESCOMPETENC)
                                       FROM GPE_HORDIAH HDH
                                      WHERE HDH.NRHORDIAM  = ?
                                        AND HDH.NRORG      = ?
                                        AND HDH.DTMESCOMPETENC <= TO_DATE(?, 'DD/MM/YYYY'))";

        $params = array(
            $this->horario->getNrhordiam(),
            $this->horario->getNrorg(),
            $this->horario->getNrhordiam(),
            $this->horario->getNrorg(),
            $dtCompetencia
        );
        $this->horarioHistorico = GpeHordiah::getObjectByWhere($where, $params);

        // LEONARDOCARVALHO - 09/12/2015 - Criada condição para feriado com marcação, quando for, tratar da mesma forma como uma dia de folga para gerar extras
        if ($tipoDiaCalend == self::FERIADO && $existeMarcacao &&
            $this->horarioHistorico->getIdtipohorario() != self::IDTIPOHORARIO_TURNO) {
            $where = "NRORG = ?
              AND IDTIPOHORARIO = 'F'
              AND DTMESCOMPETENC IN (
                                     SELECT MAX(HDH.DTMESCOMPETENC)
                                       FROM GPE_HORDIAH HDH
                                      WHERE HDH.NRORG      = ?
                                        AND IDTIPOHORARIO = 'F'
                                        AND HDH.DTMESCOMPETENC <= TO_DATE(?, 'DD/MM/YYYY')
                                        
                                        /* TRATATIVA ESPECIFICA PARA 2973 PARA NÃO GERAR SALDO EM HORARIO DSR */
                                        AND ((HDH.NRHORDIAM = 3001) OR (2973 <> ?)))
              
              /* TRATATIVA ESPECIFICA PARA 2973 PARA NÃO GERAR SALDO EM HORARIO DSR */
              AND ((NRHORDIAM = 3001) OR (2973 <> ?))
              
              AND ROWNUM <= 1";

            $params = array(
                $this->horario->getNrorg(),
                $this->horario->getNrorg(),
                $dtCompetencia,
                $this->horario->getNrorg(),
                $this->horario->getNrorg()
            );

            $this->horarioHistorico = GpeHordiah::getObjectByWhere($where, $params);

            // LEONARDOCARVALHO - 10/12/2015 - Comentado abaixo pois estava alterando o horario do dia da semana na GPE_HORARIOTURNO
            //$this->horario->setNrhordiam($this->horarioHistorico->getNrhordiam());

        }

        $where = "NRHORDIAM = ?
              AND NRORG = ?";
        $params = array(
            $this->horarioHistorico->getNrhordiam()/*$this->horario->getNrhordiam()*/,
            $this->horario->getNrorg()
        );
        $orderBy = "NRSEQINTERVHORARIO";
        $intervalosHorarioObjetos = GpeIntervaloshorario::getObjectsByWhere($where, $params, $orderBy);
        $this->endOfHorario = true;
        $intervalosHorario = array();

        /** @var \Folha\Entities\GpeIntervaloshorario $intervalos */
        foreach($intervalosHorarioObjetos as $intervalos){

            $intervalosHorario[] = array('HORA'                 => $intervalos->getHrentrada(),
                                         'DTHORABASE'           => self::retornaDataDiaOcorrencia(
                                                                                 $intervalos->getHrentrada(),
                                                                                 $data,
                                                                                 $intervalos->getIddiaocorreentrada()),
                                         'DTHRTOLERAANTES'      => self::retornaDataDiaOcorrencia(
                                                                                 $intervalos->getHrtoleraanteentrada(),
                                                                                 $data,
                                                                                 $intervalos->getIddiaocorreentrada()),
                                         'TOLERANCIAANTES'      => $intervalos->getHrtoleraanteentrada(),
                                         'DTHRTOLERAAPOS'       => self::retornaDataDiaOcorrencia(
                                                                                 $intervalos->getHrtolerapostentrada(),
                                                                                 $data,
                                                                                 $intervalos->getIddiaocorreentrada()),
                                         'TOLERANCIAAPOS'       => $intervalos-> getHrtolerapostentrada(),
                                         'HORAININUCLEO'        => $intervalos->getHrinicionucleo(),
                                         'HORAFIMNUCLEO '       => $intervalos->getHrfimnucleo(),
                                         'DIAOCORRENCIA'        => $intervalos->getIddiaocorreentrada(),
                                         'CONTROLEMARCACAO'     => $intervalos->getIdctrlmarcentrada(),
                                         'AJUSTEMARCACAO'       => $intervalos->getIdajustemarcentrada(),
                                         'HRPROJETARINTERVALO'  => $intervalos->getHrprojetarintervalo(),
                                         'DSINTERVALOPROJETADO' => $intervalos->getDsintervaloprojetado(),
                                         'IDENTRADASAIDA'       => 'E');

            $intervalosHorario[] = array('HORA'                 => $intervalos->getHrsaida(),
                                         'DTHORABASE'           => self::retornaDataDiaOcorrencia(
                                                                                 $intervalos->getHrsaida(),
                                                                                 $data,
                                                                                 $intervalos->getIddiaocorresaida()),
                                         'DTHRTOLERAANTES'      => self::retornaDataDiaOcorrencia(
                                                                                 $intervalos->getHrtoleraantesaida(),
                                                                                 $data,
                                                                                 $intervalos->getIddiaocorresaida()),
                                         'TOLERANCIAANTES'      => $intervalos->getHrtoleraantesaida(),
                                         'DTHRTOLERAAPOS'       => self::retornaDataDiaOcorrencia(
                                                                                 $intervalos->getHrtolerapostsaida(),
                                                                                 $data,
                                                                                 $intervalos->getIddiaocorresaida()),
                                         'TOLERANCIAAPOS'       => $intervalos->getHrtolerapostsaida(),
                                         'HORAININUCLEO'        => $intervalos->getHrinicionucleo(),
                                         'HORAFIMNUCLEO'        => $intervalos->getHrfimnucleo(),
                                         'DIAOCORRENCIA'        => $intervalos->getIddiaocorresaida(),
                                         'CONTROLEMARCACAO'     => $intervalos->getIdctrlmarcsaida(),
                                         'AJUSTEMARCACAO'       => $intervalos->getIdajustemarcsaida(),
                                         'HRPROJETARINTERVALO'  => $intervalos->getHrprojetarintervalo(),
                                         'DSINTERVALOPROJETADO' => $intervalos->getDsintervaloprojetado(),
                                         'IDENTRADASAIDA'       => 'S');

            $this->endOfHorario = false;
        }
        $this->intervalosHorario = $intervalosHorario;
        $this->indexHorario = 0;
    }
    
    public function getHorario(){
        return $this->horario;
    }
    
    public function setHorario($horario){
        $this->horario = $horario;
    }
    
    public function getIntervalosHorario(){
        return $this->intervalosHorario;
    }
    
    public function setIntervalosHorario($intervalosHorario){
        $this->intervalosHorario = $intervalosHorario;
    }

    public function nextHorario()
    {
        if (next($this->intervalosHorario)) {
            $this->indexHorario++;
        }
        else {
            $this->endOfHorario = true;
        }
    }

    public function prevHorario()
    {
        if(prev($this->intervalosHorario)) {
            $this->indexHorario--;
            $this->endOfHorario = false;
        }
    }

    /**
     * @return object (object)array
     */
    public function currentHorario()
    {
        /** @var $horario object */
        $horario = (object)current($this->intervalosHorario);
        return $horario;
    }

    public function resetHorario ()
    {
        reset($this->intervalosHorario);
        $this->indexHorario = 0;
        $this->endOfHorario = false;
    }

    /**
     * @return mixed
     */
    public function getIndexHorario()
    {
        return $this->indexHorario;
    }

    /**
     * @param mixed $indexHorario
     */
    public function setIndexHorario($indexHorario)
    {
        $this->indexHorario = $indexHorario;
    }

    /**
     * @return mixed
     */
    public function getEndOfHorario()
    {
        return $this->endOfHorario;
    }

    /**
     * @param mixed $endOfHorario
     */
    public function setEndOfHorario($endOfHorario)
    {
        $this->endOfHorario = $endOfHorario;
    }

    private function retornaDataDiaOcorrencia($horario, $data, $diaOcorrencia){
        $horario = DateUtil::getHoraFloatDeString($horario);
        $horario = DateUtil::getHoraStringDeFloat($horario, true);
        if($diaOcorrencia == self::DIA_SEGUINTE || $diaOcorrencia == self::FORCA_DIA_SEGUINTE){
            $data = DateUtil::subtraiIntervalo($data, 1, DateUtil::DIA);
            $data = $data->format(DateUtil::FORMATO_BRASILEIRO);
            $data.= ' '. $horario;
            $data = DateUtil::getDataDeString($data, 'd/m/Y H:i');
        }
        elseif($diaOcorrencia == self::DIA_ANTERIOR || $diaOcorrencia == self::FORCA_DIA_ANTERIOR) {
            $data = DateUtil::adicionaIntervalo($data, 1, DateUtil::DIA);
            $data = $data->format(DateUtil::FORMATO_BRASILEIRO);
            $data.= ' '. $horario;
            $data = DateUtil::getDataDeString($data, 'd/m/Y H:i');
        }
        else {
            $data = $data->format(DateUtil::FORMATO_BRASILEIRO);
            $data.= ' '. $horario;
            $data = DateUtil::getDataDeString($data, 'd/m/Y H:i');
        }

        return $data;
    }

    /**
     * @return \Folha\Entities\GpeHordiah
     */
    public function getHorarioHistorico()
    {
        return $this->horarioHistorico;
    }

    /**
     * @param \Folha\Entities\GpeHordiah $horarioHistorico
     */
    public function setHorarioHistorico($horarioHistorico)
    {
        $this->horarioHistorico = $horarioHistorico;
    }
}