/* Sistemas Integrados TEKNISA
   Gesto Consolidada

Nome: 9435_CVISTPO_ETL_CONT_GERENCIAL.sql

Descrio:
Procedure para preencher os fatos e dimenses da tabela ETLSALDCCUS 
a partir dos dados apurados do Saldo Contbil.

Criao:
16/12/2008 Leonardo Bispo

*/
CREATE OR REPLACE
PROCEDURE ETL_CONT_GERENCIAL
                     (P_DTINI  IN DATE,
                      P_DTFINA IN DATE)

IS
  V_DTFIM           DATE;
  V_DTINI           DATE;
  V_NRSEQFATCUS     VARCHAR2(12);
  V_DTMES           VARCHAR2(07);
  V_DTANO           VARCHAR2(04);
  V_BIM             VARCHAR2(6);
  V_TRI             VARCHAR2(6);
  V_SEM             VARCHAR2(6);
BEGIN
   NIVEL_ANT_CONTA;
   
    V_DTINI  := P_DTINI;

    V_DTMES  :=  TO_CHAR(V_DTINI,'MM');
    V_DTANO  :=  TO_CHAR(V_DTINI,'YYYY');

    DELETE FROM ETLSALDCCUS
    WHERE  (CDANOBI > V_DTANO) OR
          (CDANOBI = V_DTANO
          AND  CDMESBI >= V_DTMES);

    BEGIN
       SELECT NVL(MAX(LPAD(NRSEQFATCUS,12,0)),0)
       INTO   V_NRSEQFATCUS
       FROM   ETLSALDCCUS;
    EXCEPTION
    WHEN OTHERS THEN
       V_NRSEQFATCUS := '0';
    END;

    FOR C_ATUCONTA IN
        (SELECT C.CDEMPRESA
         FROM EMPRESA E, CONTOPECTBL C
         WHERE C.CDEMPRESA = E.CDEMPRESA)
    LOOP
        ATUALIZA_SALDO_GERENCIAL(C_ATUCONTA.CDEMPRESA);
    END LOOP;
    
    WHILE (V_DTINI  <=  P_DTFINA) LOOP
    
       V_DTFIM := LAST_DAY( V_DTINI);    
              
       FOR C_ETL IN
          (SELECT 
              CDEMPRESA,  CDFILIAL,   CDCENTCUST,
              CDCONTCTBL, SUM(VRSALDCCUS) AS VRSALDOCCUS
           FROM
              SALDCCUS
           WHERE 
              (TRUNC(DTSALDCCUS)  BETWEEN V_DTINI AND V_DTFIM)
           GROUP BY
              CDEMPRESA,  CDFILIAL,   CDCENTCUST,
              CDCONTCTBL
          )
       LOOP

             V_DTMES  :=  TO_CHAR(V_DTFIM,'MM');
             V_DTANO  :=  TO_CHAR(V_DTFIM,'YYYY');

             V_BIM := TRUNC ((V_DTMES - 1) / 2) + 1||' Bim';
             V_TRI := TRUNC ((V_DTMES - 1) / 3) + 1||' Tri';
             V_SEM := TRUNC ((V_DTMES - 1) / 6) + 1||' Sem';

             V_NRSEQFATCUS := LPAD(V_NRSEQFATCUS + 1, 12 ,'0');
              INSERT INTO ETLSALDCCUS
                      (NRSEQFATCUS, CDEMPRESA , CDFILIAL,
                       CDCENTCUST , CDCONTCTBL, CDMESBI,
                       BIMFATCUS  , TRIFATCUS , SEMFATCUS,
                       CDANOBI  , VRSALDCCUS)
              VALUES               
                       (V_NRSEQFATCUS,    C_ETL.CDEMPRESA,  C_ETL.CDFILIAL,
                       C_ETL.CDCENTCUST, C_ETL.CDCONTCTBL, V_DTMES,
                       V_BIM,   V_TRI,   V_SEM, 
                       V_DTANO, C_ETL.VRSALDOCCUS);
       END LOOP;

    COMMIT;
    V_DTINI := ADD_MONTHS(V_DTINI,1);
    END LOOP;
END;

/

BEGIN

UPDATE
   DIMENSAODIME
SET 
   DSPKDIME2 = 'CDCONTANTE'
WHERE 
   CDTABELADIME = 'CONTCTBL' AND
   DSPKDIME     = 'CDCONTCTBL' AND
   DSITEMDIME   = 'NMCONTCTBL';

   CRIA_FATOS('ETLSALDCCUS', 'VRSALDCCUS', 'Saldo Gerencial', 'Contbil Gerencial');

   CRIA_DIME('ETLSALDCCUS', 'CDEMPRESA',  'Empresa',         'EMPRESA', 'CDEMPRESA',   'NMEMPRESA',   NULL);
   CRIA_DIME('ETLSALDCCUS', 'CDFILIAL',   'Filial',          'FILIAL',   'CDFILIAL',   'NMFILIAL',    NULL);
   CRIA_DIME('ETLSALDCCUS', 'CDCENTCUST', 'Centro de Custo', 'CENTCUST', 'CDCENTCUST', 'NMCENTCUST',  NULL);
   CRIA_DIME('ETLSALDCCUS', 'CDCONTCTBL', 'Conta Contbil',  'CONTCTBL', 'CDCONTCTBL', 'NMCONTCTBL', 'CDCONTANTE');
   
   
   CRIA_DIME('ETLSALDCCUS', 'SEMFATCUS',    'Semestre',  NULL, NULL, NULL, NULL);
   CRIA_DIME('ETLSALDCCUS', 'TRIFATCUS',    'Trimestre', NULL, NULL, NULL, NULL);
   CRIA_DIME('ETLSALDCCUS', 'BIMFATCUS',    'Bimestre',  NULL, NULL, NULL, NULL);
   CRIA_DIME('ETLSALDCCUS', 'CDMESBI',      'Ms',       'ETLMESBI', 'CDMESBI', 'NMMESBI', NULL);
   CRIA_DIME('ETLSALDCCUS', 'CDANOBI',    'Ano',       'ETLANOBI', 'CDANOBI', 'NMANOBI', NULL);

   COMMIT;
END;
/

DECLARE
   V_CDSEQUENCIAL VARCHAR2(20) NULL;
BEGIN
    /*REGISTRANDO A PROCEDURE PARA O AGENDADOR DE EVENTOS*/
    SELECT COUNT(*)
    INTO   V_CDSEQUENCIAL
    FROM   EVENTOAEV 
    WHERE  NMOBJBDEVEN = 'ETL_CONT_GERENCIAL';
    
    IF V_CDSEQUENCIAL = 0 THEN
        BEGIN
        
            SELECT LPAD(NVL(MAX(LPAD(CDEVENTOAEV, 10, 0))+1,0),10,'0')
            INTO   V_CDSEQUENCIAL
            FROM   EVENTOAEV;
        
            INSERT INTO EVENTOAEV 
            (CDEVENTOAEV, NMEVENTOAEV, IDTPEVENAEV, CDOPERINCL, DTINCLEVEN, NMOBJBDEVEN)
            VALUES
            (V_CDSEQUENCIAL,'ETL Saldo Gerencial','STP', NULL, SYSDATE, 'ETL_CONT_GERENCIAL');
            COMMIT;
            
        EXCEPTION
           WHEN OTHERS THEN
              RAISE_APPLICATION_ERROR (-20001, 'Falha no cadastro de eventos agendveis.');
        END;
    END IF;
END;
/

DECLARE
   V_CDSEQUENCIAL VARCHAR2(20) NULL;
BEGIN
    SELECT COUNT(*)
    INTO   V_CDSEQUENCIAL
    FROM   PARAMEVENTO
    WHERE  CDEVENTOAEV = (SELECT CDEVENTOAEV
                          FROM   EVENTOAEV
                          WHERE  NMOBJBDEVEN = 'ETL_CONT_GERENCIAL');
 
    IF V_CDSEQUENCIAL = 0 THEN
        BEGIN
        
        SELECT CDEVENTOAEV
        INTO   V_CDSEQUENCIAL
        FROM   EVENTOAEV
        WHERE  NMOBJBDEVEN = 'ETL_CONT_GERENCIAL';
    
            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTINI','D','E','Data Inicial','N',0,'01');
            COMMIT;

            INSERT INTO PARAMEVENTO
            (CDEVENTOAEV,NMPARAMEVEN,IDDADPARAMEVEN,IDTIPPARAMEVEN,DSPARAMEVEN,IDPARAMNULL,VRTAMPARAM,NRSEQPARAMEVEN) 
            VALUES
            (V_CDSEQUENCIAL,'P_DTFINA','D','E','Data Final','N',0,'02');
            COMMIT;
            
        EXCEPTION
           WHEN OTHERS THEN
              RAISE_APPLICATION_ERROR (-20001, 'Falha no cadastro de eventos agendveis.');
        END;
    END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ETL_CONT_GERENCIAL','5.05.020'); END;
/